// This is a one-line comment and is ignored by the compiler.
/* This is also a comment...
 * but be careful: all lines are commented out
 * up to the end of the comment.
 */

// A macro.  Everywhere NUM_SAMPLES appears, it will be
// seen by the compiler as this number.
#define NUM_SAMPLES		25

// A global variable.
// This can be used inside any function.
int count = 1;				// Assign an initial value

// A global array.
// Initially, these will all be zero.
int samples[NUM_SAMPLES];

// Prototypes must be present before each function is used:
int sum_samples(void);		// Use void if there are no parameters.
int limit(int value, int min, int max);
void store_sample(int value);

// Execution begins at main().
// Do not call main() yourself.
int main()
{
	unsigned char x;
	int y;
	int a[10];
	
	// Assignments and expressions:
	x = 32 / 6;
	a[0] = -7;
	a[1] = x;		// x is converted to int automatically.
	y = a[0] * 2 / 5;
	a[9] = y;
	
	y = 2 / 5 * a[0];
	// y is always zero because 2 / 5 is evaluated first and is (as an integer) zero.
	
	// Calling a function:
	y = sum_samples();
	
	// Calling a function with parameters:
	samples[count] = limit(x, 10, 100);
	
	// Calling a function that doesn't return anything:
	store_sample(3);
	
	// Main is supposed to be declared int, but the return value is
	// meaningless here since there is nothing to execute after main().
	return 0;
}

int sum_samples()
{
	int i, sum;
	
	// Don't forget to set sum before you use it.
	// Its initial value is unknown!
	sum = 0;
	
	for (i = 0; i < NUM_SAMPLES; i++)
		sum = sum + samples[i];
	
	return sum;
}

int limit(int value, int min, int max)
{
	if (value < min)
	{
		// You can assign to a parameter, but it only has effect
		// inside this function.  The caller can't tell that you did this.
		value = min;
	} else if (value > max)
	{
		value = max;
	}
	
	return value;
}

void store_sample(int value)
{
	samples[count] = value;
	
	count = count + 1;
	if (count == NUM_SAMPLES)
	{
		count = 0;
	}
}
