#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <allegro.h>

#include "elevator.h"
#include "world.h"
#include "winSupport.h"

typedef struct
{
	float x1, x2, y1, y2;
	unsigned char on;
	
	char text[3];
} DisplayButton;

void drawElevator(BITMAP *buf, Elevator e, DisplayButton buttons[], int bc)
{
	float doorPos = 60 + 185 * e.doorPosition/100.0f;
	float doorEnd = 60+185;
	
	/* elevator background*/
	rectfill(buf, 30, 30, 275, 300, 0xcccccc);
	
	/* floor */
	rectfill(buf, 15, 301, 290,310, 0x403020);
	
	/* elevator opening */
	rectfill(buf, 60, 80, doorEnd, 300, 0x404040);
	
	if (doorPos > 60)
	{
		/* door */
		rectfill(buf, 60, 80, doorPos, 300, 0xffffff);
	}
	
	/* Draw the door limits */
	if (e.limit == 1)
	{
		circlefill(buf, 260, 290, 5, 0xff0000);
	}
	else if (e.limit == -1)
	{
		circlefill(buf, 45, 290, 5, 0xff0000);
	}
	
	/* lasers */
	if (e.obstacle)
	{
		int i;
		for (i=0; i<5; i++)
		{
			hline(buf, doorPos, 100 + 40*i, doorEnd, 0xff0000);
		}
	}
	else
	{
		int i, j;
		float seg = (doorEnd - doorPos)/20;
		for (i=0; i<5; i++)
		{
			for (j=0;j<20;j++)
			{
				hline(buf, doorPos + seg * j, 100 + 40*i, doorPos + seg * j + seg/2, 0x800000);
			}
		}
	}
	
	int i, color=0;
	for (i=0; i<bc; i++)
	{
		if (buttons[i].on)
		{
			color = 0xffff00;
		}
		else
		{
			color = 0xaaaa11;
		}
		
		rectfill(buf, buttons[i].x1, buttons[i].y1, buttons[i].x2, buttons[i].y2, color);
		textout_centre_ex(buf, font, buttons[i].text, buttons[i].x1 + (buttons[i].x2-buttons[i].x1)/2 +1, buttons[i].y1 + (buttons[i].y2-buttons[i].y1)/2-1, 0, -1);
	}
}

int main()
{	
	srand(time(NULL));
	int done = 0;
						   
	BITMAP *buf; /* drawing surface */
	
	Elevator e; /* the elevator */
	
	/* Initializations */
	InitWorld();
	InitElevator(&e);
	
	//set the floor button positions
	int numButtons = sizeof(e.buttons)/sizeof(e.buttons[0]);
	DisplayButton b[numButtons];
	
	int j;
	for (j=numButtons-1; j>=0; j--)
	{
		int floorButtonSize = 20;
		
		if (j == numButtons-1)
		{
			snprintf(b[j].text, sizeof(b[j].text)/sizeof(b[j].text[0]), "<>");
		}
		else if (j == numButtons-2)
		{
			snprintf(b[j].text, sizeof(b[j].text)/sizeof(b[j].text[0]), "><");
		}
		else
		{
			snprintf(b[j].text, sizeof(b[j].text)/sizeof(b[j].text[0]), "%i", j+1);
			
			b[j].x1 = 350;
			b[j].y1 = 40 + (floorButtonSize + 2)*(4-j);
			b[j].x2 = 350 + floorButtonSize;
			b[j].y2 = 40 + floorButtonSize + (floorButtonSize+2)*(4-j);
		}
		
		if (j==numButtons-1 || j==numButtons-2)
		{
			b[j].x1 = 350;
			b[j].y1 = 200 + (floorButtonSize + 2)*(4-j);
			b[j].x2 = 350 + floorButtonSize;
			b[j].y2 = 200 + floorButtonSize + (floorButtonSize+2)*(4-j);
		}
		
		b[j].on = 0;
	}
			
	/* Set up Allegro */
    allegro_init();
    install_keyboard();
	install_mouse();

	/* set up the drawing window */
    set_color_depth(32);
    if (set_gfx_mode(GFX_AUTODETECT|GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0))
    {
        printf("Failed to set graphics mode\n");
        return 1;
    }

	/* create the drawing surface */
    buf = create_bitmap(SCREEN_W, SCREEN_H);

    /* Main loop */
	int i = 0;
    while (!keypressed())
    {
		if (!e.broken && !done)
		{
			i++; /* loop counter */
			
			/* ------------ start control code ------------- */
			/* set variables for input */
			ObstacleDetected = e.obstacle;
			DoorLimit = e.limit;
			DoorCloseButton = e.buttons[4] = b[4].on;
			DoorOpenButton  = e.buttons[5] = b[5].on;
			
			int j;
			for (j=0; j<numButtons; j++)
			{
				e.buttons[j] = b[j].on;
			}
			
			/* user code */
			RunElevator(&e);
			
			b[4].on = e.buttons[4] = DoorCloseButton;
			b[5].on = e.buttons[5] = DoorOpenButton;

			/* output here with users inputs */
			StepElevator(&e);
			
			/* ------------- end control code ------------- */
			
			if (mouse_b)
			{
				int j;
				for (j=0; j<numButtons; j++)
				{
					if (mouse_x > b[j].x1 && mouse_x < b[j].x2
					&& mouse_y > b[j].y1 && mouse_y < b[j].y2)
					{
						b[j].on = 1;
					}
				}
			}
		}
		
        /* Drawing starts here*/
        clear_bitmap(buf);
		
		drawElevator(buf, e, b, sizeof(b)/sizeof(b[0]));
		
		/* debug printout */
		if (e.broken)
		{
			textprintf_ex(buf, font, 5, 5, 0xff0000, -1, "Elevator has broken! Call maintenance.");
			
		}
		else if (e.obstacle && e.doorPosition >= 85)
		{
			textprintf_ex(buf, font, 5, 5, 0xff0000, -1, "You have crushed a person. Try again.");
			done = 1;
		}
		else if (e.limit == 1 && (boarded == ELEVATOR_CAPACITY || PeopleWaiting() == 0))
		{
			//if everyone is on or the elevator is at capacity and the doors close then the task is done
			textprintf_ex(buf, font, 5, 5, 0xffffff, -1, "Congrats, everyone is onboard and the doors are closed!");
			done = 1;
		}
		else
		{
			/* only one print statement for what is happening */
			textprintf_ex(buf, font, 5, 5, 0xffffff, -1, "Door Position (%i) | Limit (%d) | Obstacle (%d) | People Waiting (%d) | %d", 
				   e.doorPosition, e.limit, e.obstacle, PeopleWaiting(), i);
		}

		/** draw the mouse */
		circle(buf, mouse_x, mouse_y, 2, 0xff00ff);
		
        vsync();    /* Does this actually do anything on Windows? */
        blit(buf, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		
		msleep(100);
    }
	
	return 0;
}
END_OF_MAIN()

