#include <stdlib.h>

#include "world.h"

/* number of steps before the motor will fail if run into a limit */
const unsigned char MOTOR_FAILURE = 5;

/* amount to step the door when powered */
const unsigned char DOOR_STEP = 4;

/* number of people wanting to board the elevator */
unsigned char people;

/* number of people that have boarded the elevator */
unsigned char boarded;

/* counter for number of total steps run */
int lifetimeCount;

/* Allow people to board if they need to 
 * Should only be called when the doors are open
 * returns non-zero if someone is boarding (aka an obstacle)*/
 
 //todo: noone boards if the doors are less than some width from fully closed
unsigned char Board()
{
	/* count for a boarding person because people don't board instantly */
	static int duration = 0;
	
	static int boardState = 0;
	
	if (people > 0)
	{	
		/* Two boarding states. board or wait to board */
		switch (boardState)
		{
			case 0:
				/* A person has a board time of 6 loops */
				if (duration >= 6)
				{
					duration = 0;
					people --;
					boarded++;
					boardState = 1;
				}
				break;
			case 1:
				/* they wait 3 loops after the last person has boarded */
				if (duration >= 3)
				{
					duration = 0;
					boardState = 0;
				}
				break;
		}
		
		duration++;
	}
	
	return !boardState;
}

void StepElevator(Elevator *e)
{
	/* count for running the motors at a limit */
	static int ovrCount = 0;
	static int boardState = 0;
	
	/* power the door */
	if (!e->broken)
	{
		e->doorMotor = DoorMotor;
		
		/* check if door is at limits and still running */
		if (e->limit && e->doorMotor)
		{
			ovrCount++;
		}
		else
		{
			ovrCount = 0;
		}
		
		if (e->doorMotor > 0)
		{
			e->doorPosition += 1 * DOOR_STEP;
		}
		else if (e->doorMotor < 0)
		{
			e->doorPosition -= 1 * DOOR_STEP;
		}
		
		/* set the limits switches */
		if (e->doorPosition <= 0)
		{
			e->doorPosition = 0;
			e->limit = -1;
		}
		else if (e->doorPosition >= 100)
		{
			e->doorPosition = 100;
			e->limit = 1;
		}
		else
		{
			e->limit = 0;
		}
		
		/* the doors have been broken by being run too long at the limit */
		if (ovrCount >= MOTOR_FAILURE)
		{
			e->broken = 1;
		}
		
		/* add another person to the queue */
		if (rand() % 40 == 0)
		{
			people++;
		}
	}
	
	/* People only begin to board if the doors are fully open
	 * people will only continue to board if the doors are more than 15% open */
	/* A person is an obstacle. */
	switch (boardState)
	{
		case 0:
			if (e->limit == -1)
			{
				boardState = 1;
			}
			break;
		case 1:
			/* if the door position is less than 85 (>15% open) or someone is already 
			 * boarding, we allow people to continue boarding */
			if (boarded < ELEVATOR_CAPACITY)
			{
				e->obstacle = Board();
				if (e->doorPosition >= 85 && !e->obstacle)
				{
					boardState = 0;
				}
			}
			break;
	}
	
	lifetimeCount++;
}

void InitElevator(Elevator *e)
{
	/* closed elevator doors at limit */
	e->doorPosition = 100;
	e->doorMotor = 0;
	e->obstacle = 0;
	e->limit = 1;
	e->broken = 0;
	
	int i;
	for (i=0; i<sizeof(e->buttons)/sizeof(e->buttons[0]); i++)
	{
		e->buttons[i] = 0;
	}
}

void InitWorld()
{
	people = rand() % (ELEVATOR_CAPACITY+1);
}

unsigned char PeopleWaiting()
{
	return people;
}
