#ifndef _GEOMETRY_H_
#define _GEOMETRY_H_

// Determinant of
// |a b|
// |c d|
#define DET(a, b, c, d) ((a) * (d) - (b) * (c))

typedef struct
{
    float x, y;
} point_t;

typedef struct
{
    point_t center;
    float radius;
} circle_t;

typedef struct
{
    point_t p[2];
} line_t;

/* Returns the distance between two points */
float point_point_distance(point_t p1, point_t p2);

/* Returns the distance between a point and a line segment */
float point_seg_distance(point_t pt, line_t seg);

/* Clips <seg> to the polygon <pts>.
 * Stores the clipped segment in <out> if it is not NULL.
 * Returns nonzero if any segment is left after clipping.
 * If the segment is completely removed by clipping, <out> is
 * unchanged and zero is returned.
 */
int clip_seg_to_poly(line_t seg, point_t *pts, int npts, line_t *out);

void clip_velocity_to_line(line_t line, point_t v, point_t *out);

/* Finds the intersection of two lines.
 * Returns nonzero if the lines intersect or zero if not.
 * The intersection is stored in *i.  If there is no intersection, *i is not modified.
 */
int line_line_intersection(line_t l1, line_t l2, point_t *i);

/* Finds the intersections of a circle and a line.
 * Returns the number of intersections (0, 1, or 2).
 * The intersections are stored in pt.  pt must be an array of two points or NULL.
 * The unused entries in pt are not modified.
 */
int circle_line_intersection(line_t line, circle_t circle, point_t *pt);

#endif /*  _GEOMETRY_H_ */
