#ifndef _WORLD_H_
#define _WORLD_H_

#include "geometry.h"

typedef struct
{
    /* The body of the robot */
    circle_t circle;

    /* The direction in which the robot is pointed, in degrees counterclockwise from +X */
    float angle;

    /* Current linear velocity */
    float velocity;

    /* Requested linear velocity */
    float set_velocity;

    /* Nonzero if the robot sees an obstacle */
    int sees_obstacle;

    /* Nonzero if the robot is colliding with the world */
    int collides;

    /* Current rotational velocity */
    float turn;

    /* Requested rotational velocity */
    float set_turn;
} robot_t;

typedef struct wall_s
{
    line_t line;

    /* Waypoint number */
    int waypoint;

    struct wall_s *next;
} wall_t;

typedef struct waypoint_s
{
    circle_t circle;

    /* Waypoint number */
    int num;

    struct waypoint_s *next;
} waypoint_t;

extern wall_t *first_wall;
extern waypoint_t *first_waypoint;

/* Highest waypoint number */
extern int last_waypoint;

/* The last waypoint to be touched. */
extern int touched_waypoint;

void wall_add(wall_t *wall);
int world_load(const char *filename);

void robot_init(robot_t *robot);
void robot_update(robot_t *robot, float dtime);

#endif /* _WORLD_H_ */
